@@ -14,6 +14,7 @@
 /* Indicate that this .c file is allowed to include the header */
 #define NPY_ITERATOR_IMPLEMENTATION_CODE
 #include "nditer_impl.h"
+#include "templ_common.h"
 
 /* Internal helper functions private to this file */
 static npy_intp
@@ -127,13 +128,23 @@ NpyIter_RemoveAxis(NpyIter *iter, int axis)
         perm[idim] = p;
     }
 
-    /* Adjust the iteration size */
-    NIT_ITERSIZE(iter) /= NAD_SHAPE(axisdata_del);
-
     /* Shift all the axisdata structures by one */
     axisdata = NIT_INDEX_AXISDATA(axisdata_del, 1);
     memmove(axisdata_del, axisdata, (ndim-1-xdim)*sizeof_axisdata);
 
+    /* Adjust the iteration size and reset iterend */
+    NIT_ITERSIZE(iter) = 1;
+    axisdata = NIT_AXISDATA(iter);
+    for (idim = 0; idim < ndim-1; ++idim) {
+        if (npy_mul_with_overflow_intp(&NIT_ITERSIZE(iter),
+                    NIT_ITERSIZE(iter), NAD_SHAPE(axisdata))) {
+            NIT_ITERSIZE(iter) = -1;
+            break;
+        }
+        NIT_ADVANCE_AXISDATA(axisdata, 1);
+    }
+    NIT_ITEREND(iter) = NIT_ITERSIZE(iter);
+
     /* Shrink the iterator */
     NIT_NDIM(iter) = ndim - 1;
     /* If it is now 0-d fill the singleton dimension */
@@ -166,6 +177,11 @@ NpyIter_RemoveMultiIndex(NpyIter *iter)
 
     itflags = NIT_ITFLAGS(iter);
     if (itflags&NPY_ITFLAG_HASMULTIINDEX) {
+        if (NIT_ITERSIZE(iter) < 0) {
+            PyErr_SetString(PyExc_ValueError, "iterator is too large");
+            return NPY_FAIL;
+        }
+
         NIT_ITFLAGS(iter) = itflags & ~NPY_ITFLAG_HASMULTIINDEX;
         npyiter_coalesce_axes(iter);
     }
@@ -349,6 +365,15 @@ NpyIter_ResetToIterIndexRange(NpyIter *iter,
     }
 
     if (istart < 0 || iend > NIT_ITERSIZE(iter)) {
+        if (NIT_ITERSIZE(iter) < 0) {
+            if (errmsg == NULL) {
+                PyErr_SetString(PyExc_ValueError, "iterator is too large");
+            }
+            else {
+                *errmsg = "iterator is too large";
+            }
+            return NPY_FAIL;
+        }
         if (errmsg == NULL) {
             PyErr_Format(PyExc_ValueError,
                     "Out-of-bounds range [%d, %d) passed to "
@@ -454,6 +479,10 @@ NpyIter_GotoMultiIndex(NpyIter *iter, npy_intp *multi_index)
     }
 
     if (iterindex < NIT_ITERSTART(iter) || iterindex >= NIT_ITEREND(iter)) {
+        if (NIT_ITERSIZE(iter) < 0) {
+            PyErr_SetString(PyExc_ValueError, "iterator is too large");
+            return NPY_FAIL;
+        }
         PyErr_SetString(PyExc_IndexError,
                 "Iterator GotoMultiIndex called with a multi-index outside the "
                 "restricted iteration range");
@@ -574,6 +603,10 @@ NpyIter_GotoIterIndex(NpyIter *iter, npy_intp iterindex)
     }
 
     if (iterindex < NIT_ITERSTART(iter) || iterindex >= NIT_ITEREND(iter)) {
+        if (NIT_ITERSIZE(iter) < 0) {
+            PyErr_SetString(PyExc_ValueError, "iterator is too large");
+            return NPY_FAIL;
+        }
         PyErr_SetString(PyExc_IndexError,
                 "Iterator GotoIterIndex called with an iterindex outside the "
                 "iteration range.");
@@ -752,8 +785,8 @@ NpyIter_IsFirstVisit(NpyIter *iter, int iop)
     if (itflags&NPY_ITFLAG_BUFFER) {
         NpyIter_BufferData *bufferdata = NIT_BUFFERDATA(iter);
         /* The outer reduce loop */
-        if (NBF_REDUCE_OUTERSTRIDES(bufferdata)[iop] == 0 &&
-                NBF_REDUCE_POS(bufferdata) != 0) {
+        if (NBF_REDUCE_POS(bufferdata) != 0 &&
+                NBF_REDUCE_OUTERSTRIDES(bufferdata)[iop] == 0) {
             return 0;
         }
     }
@@ -1298,21 +1331,20 @@ NpyIter_GetInnerFixedStrideArray(NpyIter *iter, npy_intp *out_strides)
                     out_strides[iop] = stride;
                 }
                 /*
-                 * Otherwise it's a fixed stride if the stride is 0
-                 * for all inner dimensions of the reduction double loop
+                 * Otherwise it's guaranteed to be a fixed stride if the
+                 * stride is 0 for all the dimensions.
                  */
                 else {
                     NpyIter_AxisData *axisdata = axisdata0;
-                    int idim,
-                            reduce_outerdim = NBF_REDUCE_OUTERDIM(data);
-                    for (idim = 0; idim < reduce_outerdim; ++idim) {
+                    int idim;
+                    for (idim = 0; idim < ndim; ++idim) {
                         if (NAD_STRIDES(axisdata)[iop] != 0) {
                             break;
                         }
                         NIT_ADVANCE_AXISDATA(axisdata, 1);
                     }
                     /* If all the strides were 0, the stride won't change */
-                    if (idim == reduce_outerdim) {
+                    if (idim == ndim) {
                         out_strides[iop] = stride;
                     }
                     else {
@@ -1377,7 +1409,8 @@ NpyIter_DebugPrint(NpyIter *iter)
     NpyIter_AxisData *axisdata;
     npy_intp sizeof_axisdata;
 
-    PyGILState_STATE gilstate = PyGILState_Ensure();
+    NPY_ALLOW_C_API_DEF
+    NPY_ALLOW_C_API
 
     printf("\n------ BEGIN ITERATOR DUMP ------\n");
     printf("| Iterator Address: %p\n", (void *)iter);
@@ -1598,7 +1631,7 @@ NpyIter_DebugPrint(NpyIter *iter)
     printf("------- END ITERATOR DUMP -------\n");
     fflush(stdout);
 
-    PyGILState_Release(gilstate);
+    NPY_DISABLE_C_API
 }
 
 NPY_NO_EXPORT void
@@ -1832,12 +1865,11 @@ npyiter_copy_from_buffers(NpyIter *iter)
                     *reduce_outeraxisdata = NULL;
 
     PyArray_Descr **dtypes = NIT_DTYPES(iter);
-    npy_intp transfersize = NBF_SIZE(bufferdata),
-                buffersize = NBF_BUFFERSIZE(bufferdata);
+    npy_intp transfersize = NBF_SIZE(bufferdata);
     npy_intp *strides = NBF_STRIDES(bufferdata),
              *ad_strides = NAD_STRIDES(axisdata);
     npy_intp sizeof_axisdata = NIT_AXISDATA_SIZEOF(itflags, ndim, nop);
-    char **ptrs = NBF_PTRS(bufferdata), **ad_ptrs = NAD_PTRS(axisdata);
+    char **ad_ptrs = NAD_PTRS(axisdata);
     char **buffers = NBF_BUFFERS(bufferdata);
     char *buffer;
 
